#!/usr/bin/perl

# (C) 2006 by Oli Sennhauser <oli.sennhauser@bluewin.ch>.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA
# 02110-1301, USA.

use strict;
use warnings;

use DBI;
use Getopt::Long;

my %lError = (
    'OK'       => 0
  , 'Warning'  => 1
  , 'Critical' => 2
  , 'Unknown'  => 3
);

# Parameters
# ----------

my $pUser = '';
my $pPassword = '';
my $pHost = 'localhost';
my $pPort = '3306';

my $rc = GetOptions(
    'user|u=s' => \$pUser
  , 'password|p=s' => \$pPassword
  , 'host|h=s' => \$pHost
  , 'port=i' => \$pPort
);

if ( ! $rc ) {
  print("Error in parameters. User = $pUser, Password=hidden, Host = $pHost, Port = $pPort");
  exit($lError{'Unknown'});
}

if( @ARGV != 0 ) {
  print("Error in parameters. To many arguments: @ARGV");
  exit($lError{'Unknown'});
}

# Start here
# ----------

my $cmd = "mysqladmin --user=$pUser --host=$pHost --port=$pPort --password=$pPassword --protocol=tcp ping 2>/dev/null";

`$cmd`;
$rc = $?;

if ( $rc ) {
  print("Error. mysqladmin ping failed to connect.");
  exit($lError{'Critical'});
}

my ($dbh, $sql, $sth);

$dbh = DBI->connect("DBI:mysql::$pHost:$pPort"
                  , $pUser, $pPassword
                  , { RaiseError => 0
                    , PrintError => 0
                    , AutoCommit => 0
                    }
                   );

if ( ! $dbh ) {

  if ( $DBI::errstr =~ m/Can't connto to/ ) {
    print("Error during connection");
    exit($lError{'Critical'});
  }

  if ( $DBI::errstr =~ m/Access denied for user/ ) {
    print("User does not have access privileges to database.");
    exit($lError{'Warning'});
  }

  print("Error during connection");
  exit($lError{'Critical'});
}

$sql = "
  SELECT USER()
";

my $lUser;

$sth = $dbh->prepare( $sql );
if ( $DBI::err ) {
  print("Error in preparing");
  $dbh->disconnect;
  exit($lError{'Critical'});
}

$sth->execute();
if ( $sth->err ) {
  print("Error in executing");
  $dbh->disconnect;
  exit($lError{'Critical'});
}

$sth->bind_columns(undef, \$lUser);
if ( $sth->err ) {
  print("Error in binding");
  $sth->finish;
  $dbh->disconnect;
  exit($lError{'Critical'});
}

$sth->fetchrow_arrayref();
$sth->bind_columns(undef, \$lUser);
if ( $sth->err ) {
  print("Error in fetchting");
  $sth->finish;
  $dbh->disconnect;
  exit($lError{'Critical'});
}

$sth->finish;
$dbh->disconnect;

print "Database seems up and running...";
exit($lError{'OK'});
